package com.softmoore.util;


/**
 * A simple "stopwatch" timer class with millisecond accuracy.<br>
 * Usage Notes:<br>
 * 1.  Timer is started automatically when constructed.<br>
 * 2.  Method start() restarts the timer.<br>
 * 3.  Method stop() can only be called if the timer is currently running
 *     (i.e., if the timer is not already stopped).<br>
 * 4.  A single timer can be restarted/stopped multiple times.<br>
 * 5.  Methods that return the elapsed time (including the toString() method)
 *     can only be called if the timer is stopped.<br>
 */
class StopWatch
  {

    var startTime : Long = System.currentTimeMillis();
    var stopTime  : Long = 0;

    var isRunning : Boolean = true;


    /**
     * Sets the start time for the StopWatch to the current time.
     */
    def start()
      {
        startTime = System.currentTimeMillis();
        isRunning = true;
      }


    /**
     * Sets the stop time for the StopWatch to the current time.
     */
    def stop()
      {
        if (!isRunning)
          {
            val message : String = "com.softmoore.util.Timer:  " +
                "method stop() can only be called if the timer is currently running";
            throw new IllegalStateException(message);
          }

        stopTime = System.currentTimeMillis();

        isRunning = false;
      }


    /**
     * Returns the elapsed time in milliseconds
     */
    def getElapsedTimeInMilliseconds() : Long =
      {
        if (isRunning)
          {
            val message :String = "com.softmoore.util.Timer:  " +
                "can only be called if the timer has been stopped";
            throw new IllegalStateException(message);
          }

        return stopTime - startTime;
      }


    /**
     * Returns the elapsed time in seconds
     */
    def getElapsedTimeInSeconds() : Double =
      {
        return (getElapsedTimeInMilliseconds()/1000.0).toDouble;
      }


    /**
     * Returns the elapsed time in minutes
     */
    def getElapsedTimeInMinutes() : Double =
      {
        return getElapsedTimeInSeconds()/60.0;
      }


    /**
     * Returns the elapsed time in hours
     */
    def getElapsedTimeInHours() : Double =
      {
        return getElapsedTimeInMinutes()/60.0;
      }


    override 
    def toString() : String =
      {
        return getElapsedTimeInMilliseconds().toString() + " ms";
      }


    /**
     * Returns the start time in milliseconds
     */
    def getStartTime() : Long =
      {
        return startTime;
      }


    /**
     * Returns the stop time in milliseconds
     */
    def getStopTime() : Long =
      {
        return stopTime;
      }
  }
